@@ -133,7 +133,12 @@ void display_help_menu()
    "\n"
    "In addition, the program allows the output to be formatted for inclusion\n"
    "into an AFNI dataset's header, whence it can be used in the AFNI Clusterize\n"
-   "interface to show approximate alpha values for the displayed clusters.\n"
+   "interface to show approximate alpha values for the displayed clusters, where\n"
+   "the per-voxel p-value is taken from the interactive threshold slider in the\n"
+   "AFNI 'Define Overlay' control panel, and then the per-cluster alpha value\n"
+   "is interpolated in this table from 3dClustSim.  As you change the threshold\n"
+   "slider, the per-voxel p-value (shown below the slider) changes, and then\n"
+   "the interpolated alpha values are updated.\n"
    "\n"
    "Three different clustering methods can be used -- the program can give\n"
    "you results for any or all of them in one run -- see the '-NN' option.\n"
@@ -317,6 +322,17 @@ void display_help_menu()
    "  sub-brick is a statistical parameter (e.g., a t- or F-statistic), from which\n"
    "  a per-voxel p-value can be calculated, so that Clusterize can interpolate\n"
    "  in the C(p,alpha) table.\n"
+   " ++ To be clear, the per-voxel p-value is taken from the AFNI GUI threshold\n"
+   "    slider (the p-value is shown beneath the slider), and then the C(p,alpha)\n"
+   "    table is inverse-interpolated to find the per-cluster alpha value for\n"
+   "    each different cluster size.\n"
+   " ++ As you move the AFNI threshold slider, the per-voxel (uncorrected for\n"
+   "    multiple comparisons) p-value changes, the cluster sizes change (as fewer\n"
+   "    or more voxels are included), and so the reported per-cluster alpha\n"
+   "    values change for both reasons -- different p and different cluster size.\n"
+   " ++ The alpha values reported are 'per-cluster', and are not themselves\n"
+   "    corrected for multiple comparisons ACROSS clusters.  These alpha values\n"
+   "    are corrected for multiple comparisons within a cluster.\n"
    "\n"
    "* AFNI will use the NN1, NN2, NN3 tables as needed in its Clusterize\n"
    "  interface if they are all stored in the statistics dataset header,\n"
@@ -1125,6 +1141,8 @@ int main( int argc , char **argv )
 
   /*----- does user request help menu? -----*/
 
+  AFNI_SETUP_OMP(0) ;  /* 24 Jun 2013 */
+
   if( argc < 2 || strcmp(argv[1],"-help") == 0 ) display_help_menu() ;
 
   /*----- get the list of things to do -----*/
