@@ -268,7 +268,7 @@ typedef struct {
       int        func_resam_mode , anat_resam_mode , pts_color ;
       int        thr_resam_mode ;               /* 09 Dec 1997 */
       int        thr_onoff ;                    /* 28 Jun 2007 */
-      int        thr_olay1 ;                    /* 13 Aug 2010 */
+      int        thr_olayx ;                    /* 13 Aug 2010 */
       int        thr_sign ;                     /* 08 Aug 2007 */
 
       /* 3/24/95: range data for conversion of pbar
@@ -305,7 +305,7 @@ typedef struct {
 
       int   i1_icor , j2_icor , k3_icor;  /* for InstaCorr -- 08 May 2009 */
       float xi_icor , yj_icor , zk_icor ; /* DICOM coords -- 17 Mar 2010 */
-      
+
       float *th_sort;  /* sorted values of overlay threshold */
       int N_th_sort; /* number of values stored in th_sort */
       char  th_sortid[256]; /* indentifier of provenance of th_sort */
@@ -402,7 +402,7 @@ typedef struct {
       Widget pop_whereami_pb , pop_ttren_pb ;
       MCW_textwin *pop_whereami_twin ;
       MCW_htmlwin *pop_whereami_htmlwin ;
-      
+
       Widget pop_sumato_pb ;
       Widget pop_mnito_pb ;  /* 01 May 2002 */
 
@@ -411,6 +411,8 @@ typedef struct {
 
       Widget pop_instacorr_pb ;   /* 06 May 2009 */
       Widget pop_icorrjump_pb ;
+      Widget pop_icorrapair_pb;   /* Apr 2013 */
+      Widget pop_icorramirr_pb;
 } AFNI_imaging_widgets ;
 
 /*--- 19 Aug 2002: Switch Surface control box ---*/
@@ -434,6 +436,7 @@ typedef struct {
   Widget top_lab;           /* overall report text */
   Widget top_menu , histrange_pb ;
   MCW_bbox *histsqrt_bbox ;
+  MCW_bbox *spearman_bbox ; /* 02 Jan 2013 */
 
   MCW_bbox *usemask_bbox ;  /* zero-th row of controls [01 Aug 2011] */
 
@@ -571,7 +574,7 @@ extern void AFNI_sesslab_EV( Widget, XtPointer, XEvent *, Boolean * ) ; /* 30 Ap
 extern void flush_vinfo_sort(AFNI_view_info *vinfo, char *sel);/*ZSS: 04/26/12*/
 extern void flush_3Dview_sort(struct Three_D_View *im3d, char *sel);
 extern float *get_3Dview_sort(struct Three_D_View *im3d, char *sel);
-extern float  get_3Dview_func_thresh(struct Three_D_View *im3d, 
+extern float  get_3Dview_func_thresh(struct Three_D_View *im3d,
                                      int apply_power);
 extern float AFNI_thresh_from_percentile(struct Three_D_View *im3d, float perc);
 
@@ -662,7 +665,7 @@ typedef struct {
 
       Widget thr_menu ;
       MCW_bbox *thr_onoff_bbox ;
-      MCW_bbox *thr_olay1_bbox ;
+      MCW_bbox *thr_olayx_bbox ;
       Widget thr_autothresh_pb ;
       MCW_arrowval *thr_sign_av ;  /* 08 Aug 2007 */
       Widget thr_fdr_pb ;          /* 29 Jan 2008 */
@@ -700,8 +703,8 @@ typedef struct {
 
       Widget gicor_rowcol , gicor_pb , gicor_label ; /* 22 Dec 2009 */
 
-      Widget         buck_frame , buck_rowcol ;
-      MCW_arrowval * anat_buck_av , *fim_buck_av , *thr_buck_av ;  /* 30 Nov 1997 */
+      Widget        buck_frame , buck_rowcol ;
+      MCW_arrowval *anat_buck_av , *fim_buck_av , *thr_buck_av ;  /* 30 Nov 1997 */
 
       Widget range_frame , range_rowcol , range_label ;
       MCW_bbox     *range_bbox ;
@@ -866,6 +869,7 @@ typedef struct {
 
    Widget hidden_mission_pb ;  /* 06 Jun 2001 */
    Widget hidden_gamberi_pb ;  /* 14 Oct 2003 */
+   Widget hidden_hbmjust_pb ;  /* 05 Jun 2013 */
    Widget hidden_ranpoem_pb ;  /* 15 Oct 2003 */
    Widget hidden_speech_pb  ;  /* 25 Nov 2003 */
    Widget hidden_faces_pb   ;  /* 17 Dec 2004 */
@@ -1115,6 +1119,12 @@ typedef struct Three_D_View {
       int cont_perc_thr;       /* ZSS percentile thresholding. April 26 2012 */
 } Three_D_View ;
 
+#define IM3D_CLEAR_TMASK(iq)                                                                   \
+ do{ CLEAR_TMASK((iq)->b123_anat); CLEAR_TMASK((iq)->b231_anat); CLEAR_TMASK((iq)->b312_anat); \
+     CLEAR_TMASK((iq)->b123_fim) ; CLEAR_TMASK((iq)->b231_fim) ; CLEAR_TMASK((iq)->b312_fim) ; \
+     CLEAR_TMASK((iq)->b123_ulay); CLEAR_TMASK((iq)->b231_ulay); CLEAR_TMASK((iq)->b312_ulay); \
+ } while(0)
+
 /*! Force re-volume-editing when this viewer is redisplayed */
 
 #define IM3D_VEDIT_FORCE(iq) (iq)->vedset.flags=1
@@ -1165,9 +1175,38 @@ extern void CLU_setup_alpha_tables( Three_D_View * ) ; /* Jul 2010 */
 
 /*! Change InstaCorr popup buttons status */
 
-#define SENSITIZE_INSTACORR(iq,bb)                           \
+#define SENSITIZE_INSTACORR_INDIV(iq,bb)                     \
  do{ XtSetSensitive((iq)->vwid->imag->pop_instacorr_pb,bb) ; \
      XtSetSensitive((iq)->vwid->imag->pop_icorrjump_pb,bb) ; \
+     XtUnmanageChild((iq)->vwid->imag->pop_icorrapair_pb) ;  \
+     XtUnmanageChild((iq)->vwid->imag->pop_icorramirr_pb) ;  \
+ } while(0)
+
+#define SENSITIZE_INSTACORR_GROUP(iq,bb)                        \
+ do{ GICOR_setup *gs = (iq)->giset ;                            \
+     int ap_allow  = GICOR_apair_allow_bit(gs) ;                \
+     int ap_ready  = GICOR_apair_ready_bit(gs) ;                \
+     int ap_mirror = GICOR_apair_mirror_bit(gs);                \
+     int bp = (bb) && (!ap_allow || ap_ready || ap_mirror) ;    \
+     if( ap_allow ){                                            \
+       int ba = (bb) && ap_allow && !ap_mirror ;                \
+       XtManageChild((iq)->vwid->imag->pop_icorrapair_pb) ;     \
+       XtManageChild((iq)->vwid->imag->pop_icorramirr_pb) ;     \
+       XtSetSensitive((iq)->vwid->imag->pop_icorrapair_pb,ba) ; \
+       XtSetSensitive((iq)->vwid->imag->pop_icorramirr_pb,bb) ; \
+       MCW_set_widget_bg((iq)->vwid->imag->pop_icorramirr_pb,   \
+                         (ap_mirror) ? "white" : "black" , 0 ); \
+     } else {                                                   \
+       XtUnmanageChild((iq)->vwid->imag->pop_icorrapair_pb) ;   \
+       XtUnmanageChild((iq)->vwid->imag->pop_icorramirr_pb) ;   \
+     }                                                          \
+     XtSetSensitive((iq)->vwid->imag->pop_instacorr_pb,bp) ;    \
+     XtSetSensitive((iq)->vwid->imag->pop_icorrjump_pb,bp) ;    \
+ } while(0)
+
+#define SENSITIZE_INSTACORR(iq,bb)                                  \
+ do{ if( (iq)->giset != NULL ){ SENSITIZE_INSTACORR_GROUP(iq,bb); } \
+     else                     { SENSITIZE_INSTACORR_INDIV(iq,bb); } \
  } while(0)
 
 /*! Allow InstaCorr in this viewer */
@@ -1383,6 +1422,9 @@ extern "C" {
 
 extern void GICOR_setup_func(NI_stream, NI_element *) ;        /* 22 Dec 2009 */
 extern void GICOR_process_dataset( NI_element *nel, int ct ) ; /* 23 Dec 2009 */
+extern void GICOR_process_message( NI_element *nel ) ;            /* Apr 2013 */
+extern void process_NIML_textmessage( NI_element * ) ;            /* Apr 2013 */
+
 
 extern void ICALC_make_widgets( Three_D_View *im3d ) ;   /* 18 Sep 2009 */
 
@@ -1451,7 +1493,7 @@ typedef struct {
 
    int ijk_lock ;                                /* 11 Sep 2000 */
    int thr_lock ;
-   
+
    THD_session *session ;                        /* 20 Dec 2001 */
 
    MCW_function_list registered_slice_proj ;     /* 31 Jan 2002 */
@@ -1598,7 +1640,7 @@ extern int AFNI_clus_find_xyz( Three_D_View *im3d , float x,float y,float z ) ;
 extern void AFNI_clus_action_CB( Widget w , XtPointer cd , XtPointer cbs ) ;
 
 extern void AFNI_update_dataset_viewing( THD_3dim_dataset * ); /* 21 Jul 2009 */
-extern void AFNI_alter_wami_text(Three_D_View *im3d, char *utlab); 
+extern void AFNI_alter_wami_text(Three_D_View *im3d, char *utlab);
 
 #define AFNI_SEE_FUNC_ON(iq) ( MCW_set_bbox( (iq)->vwid->view->see_func_bbox, 1 ), \
                                AFNI_see_func_CB( NULL , (XtPointer)(iq) , NULL )  )
@@ -1658,7 +1700,7 @@ extern int  AFNI_jumpto_ijk          ( Three_D_View * , int, int, int  ) ;
 extern void AFNI_jumpto_ijk_CB       ( Widget , XtPointer , MCW_choose_cbs * ) ;
 extern void AFNI_sumato_CB           ( Widget , XtPointer , MCW_choose_cbs * ) ;
 extern void AFNI_mnito_CB            ( Widget , XtPointer , MCW_choose_cbs * ) ;
-extern void AFNI_check_obliquity     ( Widget , THD_3dim_dataset * , 
+extern void AFNI_check_obliquity     ( Widget , THD_3dim_dataset * ,
                                                 THD_3dim_dataset * ) ;
 
 extern void AFNI_crosshair_pop_CB    ( Widget , XtPointer , XtPointer ) ; /* 12 Mar 2004 */
@@ -1752,6 +1794,7 @@ extern void   AFNI_icor_setref_locked( Three_D_View *im3d ) ;      /* 15 May 200
 extern int    AFNI_icor_setref_anatijk( Three_D_View *, int,int,int ) ;      /* 17 Mar 2010 */
 extern int    AFNI_icor_setref_xyz    ( Three_D_View *, float,float,float );
 extern int    AFNI_gicor_setref_xyz   ( Three_D_View *, float,float,float ); /* 23 Dec 2009 */
+extern void   AFNI_gicor_setapair_xyz ( Three_D_View *, float,float,float );    /* Apr 2013 */
 
 extern Boolean AFNI_refashion_dataset( Three_D_View * ,
                                        THD_3dim_dataset *, THD_dataxes * , int ) ;
