@@ -41,7 +41,7 @@ static AFNI_friend afni_friends[] = {
   { "A Jesmanowicz"  , (             8 |      32                        ) } ,
   { "MS Beauchamp"   , ( 1 | 2 | 4 | 8 | 16 | 32      | 128             ) } ,
   { "JA Bobholz"     , (             8 | 16 | 32      | 128             ) } ,
-  { "JA Frost"       , (             8 | 16                             ) } ,
+  { "JA Frost"       , (             8 | 16           | 128             ) } ,
   { "J Kummer"       , (         4 | 8      | 32                        ) } ,
   { "BD Ward"        , (         4 | 8           | 64       | 512       ) } ,
   { "S Marrett"      , (             8 | 16                             ) } ,
@@ -95,7 +95,10 @@ static AFNI_friend afni_friends[] = {
   { "W Gaggl"        , (         4 | 8                                  ) } ,
   { "V Hradil"       , (         4 | 8 | 16                             ) } ,
   { "D Handwerker"   , (         4 | 8                                  ) } ,
-  { "J Gonzalez-Castillo" , (    4 | 8                                  ) }
+  { "J Gonzalez-Castillo" , (    4 | 8                | 128             ) } ,
+  { "J Evans"        ,      (    4 | 8                | 128             ) } ,
+  { "S Inati"        ,      (    4 | 8                                  ) } ,
+  { "P Taylor"       ,      (                      64                   ) }
 } ;
 
 #define NUM_FRIENDS (sizeof(afni_friends)/sizeof(AFNI_friend))
@@ -245,7 +248,6 @@ static mday holiday[] = {
    {FEB,12,"Charles Darwin's birthday"                               } ,
    {FEB,13,"Kim Novak's birthday"                                    } ,
    {FEB,14,"Saint Valentine's Day"                                   } ,
-   {FEB,15,"Anniversary of AFNI's release!"                          } ,
    {FEB,15,"Galileo Galilei's birthday"                              } ,
    {FEB,18,"Gambia Independence Day"                                 } ,
    {FEB,19,"Nikolaus Kopernikus' birthday"                           } ,
@@ -270,8 +272,10 @@ static mday holiday[] = {
    {MAR, 5,"Anniversary of Boston Massacre"                          } ,
    {MAR, 6,"Michelangelo Buonarroti's birthday"                      } ,
    {MAR, 8,"Kenneth Grahame's birthday"                              } ,
+   {MAR, 8,"International Women's Day"                               } ,
+   {MAR,11,"World Day of Muslim Culture"                             } ,
    {MAR,12,"Kemal Ataturk's birthday"                                } ,
-   {MAR,12,"Mauritius  Independence Day"                             } ,
+   {MAR,12,"Mauritius Independence Day"                              } ,
    {MAR,14,"Albert Einstein's birthday"                              } ,
    {MAR,15,"The Ides of March"                                       } ,
    {MAR,16,"James Madison's birthday"                                } ,
@@ -281,7 +285,9 @@ static mday holiday[] = {
    {MAR,19,"Saint Joseph's Day"                                      } ,
    {MAR,19,"Liechtenstein National Day"                              } ,
    {MAR,20,"Anniversary of Uncle Tom's Cabin"                        } ,
+   {MAR,20,"International Day of Happiness"                          } ,
    {MAR,21,"Jean Baptiste Joseph Fourier's birthday"                 } ,
+   {MAR,21,"World Poetry Day"                                        } ,
    {MAR,23,"Emmy Noether's birthday"                                 } ,
    {MAR,23,"Pakistan Day"                                            } ,
    {MAR,24,"Anniversary of Kosciusko's Oath to Nation"               } ,
@@ -309,6 +315,7 @@ static mday holiday[] = {
    {APR,11,"Dean Acheson's birthday"                                 } ,
    {APR,12,"Anniversary of Yuri Gagarin's spaceflight"               } ,
    {APR,13,"Thomas Jefferson's birthday"                             } ,
+   {APR,13,"New Year's Day (SE Asia)"                                } ,
    {APR,14,"Anniversary of Lincoln's assasination"                   } ,
    {APR,15,"Anniversary of sinking of Titanic"                       } ,
    {APR,15,"Arnold Toynbee's birthday"                               } ,
@@ -341,11 +348,14 @@ static mday holiday[] = {
    {MAY, 1,"Emperor Claudius's birthday"                             } ,
    {MAY, 1,"Herman Melville's birthday"                              } ,
    {MAY, 1,"Anniversary of Crab Nebula Supernova"                    } ,
+   {MAY, 1,"May Day"                                                 } ,
+   {MAY, 1,"Bealtaine"                                               } ,
    {MAY, 2,"Baron Von Richtofen's birthday"                          } ,
    {MAY, 3,"Anniversary of Polish Constitution"                      } ,
    {MAY, 3,"Nicolo Machiavelli's birthday"                           } ,
    {MAY, 3,"Golda Meir's birthday"                                   } ,
    {MAY, 4,"Audrey Hepburn's birthday"                               } ,
+   {MAY, 5,"Cinco de Mayo"                                           } ,
    {MAY, 5,"Henryk Sienkiewicz's birthday"                           } ,
    {MAY, 6,"Anniversary of Hindenburg explosion"                     } ,
    {MAY, 7,"Anniversary of sinking of Lusitania"                     } ,
@@ -353,6 +363,7 @@ static mday holiday[] = {
    {MAY, 8,"Harry S Truman's birthday"                               } ,
    {MAY, 9,"John Brown's birthday"                                   } ,
    {MAY, 9,"Anton Cermak's birthday"                                 } ,
+   {MAY, 9,"Europe Day"                                              } ,
    {MAY,10,"Gustav Stresemann's birthday"                            } ,
    {MAY,10,"Fred Astaire's birthday"                                 } ,
    {MAY,11,"Laos Constitution Day"                                   } ,
@@ -380,6 +391,7 @@ static mday holiday[] = {
    {MAY,28,"Ian Fleming's birthday"                                  } ,
    {MAY,29,"John F Kennedy's birthday"                               } ,
    {MAY,29,"Harry Bateman's birthday"                                } ,
+   {MAY,29,"Anniversary of first Everest summit"                     } ,
    {MAY,30,"St. Jeanne d'Arc's day"                                  } ,
    {MAY,31,"Walt Whitman's birthday"                                 } ,
    {MAY,31,"South Africa Republic Day"                               } ,
@@ -441,6 +453,7 @@ static mday holiday[] = {
    {JUL, 6,"Malawi Independence Day"                                 } ,
    {JUL, 7,"Satchel Paige's birthday"                                } ,
    {JUL, 7,"Robert Heinlein's birthday"                              } ,
+   {JUL, 7,"Anniversary of Sliced Bread!"                            } ,
    {JUL, 9,"Argentina Independence Day"                              } ,
    {JUL, 9,"Palau Constitution Day"                                  } ,
    {JUL,10,"Bahamas Independence Day"                                } ,
@@ -452,16 +465,18 @@ static mday holiday[] = {
    {JUL,14,"Iraq Republic Day"                                       } ,
    {JUL,14,"Gerald Ford's birthday"                                  } ,
    {JUL,15,"Anniversary of Battle of Grunwald"                       } ,
+   {JUL,15,"Rembrandt van Rijn's birthday"                           } ,
    {JUL,16,"Anniversary of 1st atomic explosion"                     } ,
    {JUL,16,"Anniversary of Apollo 11's launch"                       } ,
    {JUL,17,"Anniversary of Disneyland's opening"                     } ,
    {JUL,17,"South Korea Constitution Day"                            } ,
    {JUL,18,"John Glenn's birthday"                                   } ,
    {JUL,18,"Nelson Mandela's birthday"                               } ,
+   {JUL,18,"Anniversary of Assault on Ft Wagner"                     } ,
    {JUL,19,"George McGovern's birthday"                              } ,
    {JUL,19,"Edgar Degas' birthday"                                   } ,
    {JUL,20,"Anniversary of Apollo 11 Moon landing"                   } ,
-   {JUL,20,"Edmund Hillary's birthday"                               } ,
+   {JUL,20,"Sir Edmund Hillary's birthday"                           } ,
    {JUL,20,"Columbia Independence Day"                               } ,
    {JUL,21,"Belgium Independence Day"                                } ,
    {JUL,21,"Martyr's Day in Bolivia"                                 } ,
@@ -490,7 +505,9 @@ static mday holiday[] = {
    {AUG, 1,"Dahomey Independence Day"                                } ,
    {AUG, 1,"Switzerland Confederation Day"                           } ,
    {AUG, 1,"Jerry Garcia's birthday"                                 } ,
+   {AUG, 1,"IPA Day - bottoms up!"                                   } ,
    {AUG, 2,"Anniversary of Battle of Cannae"                         } ,
+   {AUG, 3,"International Beer Day!"                                 } ,
    {AUG, 4,"Percy Bysshe Shelley's birthday"                         } ,
    {AUG, 4,"Founding of US Coast Guard (1790)"                       } ,
    {AUG, 5,"Upper Volta Independence Day"                            } ,
@@ -546,6 +563,8 @@ static mday holiday[] = {
    {SEP, 8,"Anniversary of Star Trek TV debut"                       } ,
    {SEP, 8,"Andorra National Festival"                               } ,
    {SEP, 9,"Bulgaria Liberation Day"                                 } ,
+   {SEP,11,"9/11"                                                    } ,
+   {SEP,11,"September Eleven"                                        } ,
    {SEP,12,"Anniversary of Battle of Vienna"                         } ,
    {SEP,14,"Jan Masaryk's birthday"                                  } ,
    {SEP,14,"Anniversary of Friston & Cox Debut"                      } ,
@@ -769,6 +788,113 @@ static mday holiday[] = {
    {DEC,31,"George C Marshall's birthday"                            } ,
    {DEC,31,"Simon Wiesenthal's birthday"                             } ,
 
+   /* some other stuff from Wikipedia */
+
+   {JAN,  1, "Global Family Day"   } ,
+   {JAN,  1, "World Day of Peace"   } ,
+   {JAN, 31, "Street Children's Day"   } ,
+   {FEB,  2, "World Wetlands Day"   } ,
+   {FEB,  4, "World Cancer Day"   } ,
+   {FEB, 11, "World Day of the Sick"   } ,
+   {FEB, 13, "World Radio Day"   } ,
+   {FEB, 20, "World Day of Social Justice"   } ,
+   {MAR, 14, "Pi Day (celebrate at 4pm)"   } ,
+   {MAR, 20, "World Sparrow Day"   } ,
+   {MAR, 21, "International Day of Nowruz"   } ,
+   {MAR, 21, "World Down Syndrome Day"   } ,
+   {MAR, 21, "World Puppetry Day"   } ,
+   {MAR, 21, "International Day of Forests"   } ,
+   {MAR, 22, "World Water Day"   } ,
+   {MAR, 23, "World Meteorological Day"   } ,
+   {MAR, 24, "World Tuberculosis Day"   } ,
+   {MAR, 31, "International Cigar Day"   } ,
+   {APR,  2, "World Autism Awareness Day"   } ,
+   {APR,  7, "World Health Day"   } ,
+   {APR,  8, "International Romani Day"   } ,
+   {APR, 16, "World Entrepreneurship Day"   } ,
+   {APR, 17, "World Hemophilia Day"   } ,
+   {APR, 22, "International Mother Earth Day"   } ,
+   {APR, 24, "World Day for Laboratory Animals"   } ,
+   {APR, 25, "World Malaria Day"   } ,
+   {APR, 30, "International Jazz Day"   } ,
+   {MAY,  5, "Day of Vesak"   } ,
+   {MAY,  5, "International Midwives Day"   } ,
+   {MAY, 12, "World Migratory Bird Day"   } ,
+   {MAY, 12, "International Nurses Day"   } ,
+   {MAY, 15, "International Day of Families"   } ,
+   {MAY, 18, "International Museum Day"   } ,
+   {MAY, 22, "World Goth Day"   } ,
+   {MAY, 23, "World Turtle Day"   } ,
+   {MAY, 25, "Geek Pride Day"   } ,
+   {MAY, 31, "World No-Tobacco Day"   } ,
+   {JUN,  1, "International Children's Day"   } ,
+   {JUN,  5, "World Environment Day"   } ,
+   {JUN,  8, "World Oceans Day"   } ,
+   {JUN,  8, "World Brain Tumor Day"   } ,
+   {JUN, 10, "Ballpoint Pen Day"   } ,
+   {JUN, 14, "World Blood Donor Day"   } ,
+   {JUN, 18, "Autistic Pride Day"   } ,
+   {JUN, 19, "World Sickle Cell Day"   } ,
+   {JUN, 20, "World Refugee Day"   } ,
+   {JUN, 21, "World Music Day"   } ,
+   {JUN, 23, "International Widow's Day"   } ,
+   {JUN, 25, "Day of the Seafarer"   } ,
+   {JUL,  2, "World UFO Day"   } ,
+   {JUL,  8, "Writer's Day"   } ,
+   {JUL, 11, "World Population Day"   } ,
+   {JUL, 29, "International Tiger Day"   } ,
+   {AUG,  5, "International Beer Day"   } ,
+   {AUG,  7, "International Hipster Day"   } ,
+   {AUG, 12, "International Youth Day"   } ,
+   {AUG, 13, "International Lefthanders Day"   } ,
+   {AUG, 19, "World Humanitarian Day"   } ,
+   {AUG, 19, "World Photography Day"   } ,
+   {AUG, 21, "World Fashion Day"   } ,
+   {AUG, 24, "World Hip Hop Day"   } ,
+   {SEP,  8, "International Literacy Day"   } ,
+   {SEP,  8, "World Physical Therapy Day"   } ,
+   {SEP, 19, "Talk Like a Pirate Day"   } ,
+   {SEP, 20, "International Day of Freedom"   } ,
+   {SEP, 21, "International Day of Peace"   } ,
+   {SEP, 21, "World Alzheimer's Day"   } ,
+   {SEP, 22, "World Rhino Day"   } ,
+   {SEP, 27, "World Tourism Day"   } ,
+   {SEP, 28, "World Rabies Day"   } ,
+   {SEP, 30, "World Helping Day"   } ,
+   {SEP, 30, "Blasphemy Day"   } ,
+   {OCT,  1, "World Vegetarian Day"   } ,
+   {OCT,  2, "International Day of Non-Violence"   } ,
+   {OCT,  4, "World Animal Day"   } ,
+   {OCT,  5, "World Teachers' Day"   } ,
+   {OCT,  8, "World Humanitarian Action Day"   } ,
+   {OCT,  9, "World Post Day"   } ,
+   {OCT, 10, "World Mental Health Day"   } ,
+   {OCT, 12, "World Arthritis Day"   } ,
+   {OCT, 14, "World Standards Day"   } ,
+   {OCT, 15, "Global Handwashing Day"   } ,
+   {OCT, 16, "World Food Day"   } ,
+   {OCT, 20, "World Osteoporosis Day"   } ,
+   {OCT, 20, "World Statistics Day"   } ,
+   {OCT, 24, "United Nations Day"   } ,
+   {OCT, 28, "INTERNATIONAL CAPS LOCK DAY"   } ,
+   {OCT, 28, "International Animation Day"   } ,
+   {NOV, 12, "World Pneumonia Day"   } ,
+   {NOV, 13, "World Kindness Day"   } ,
+   {NOV, 14, "World Diabetes Day"   } ,
+   {NOV, 15, "Day of the Imprisoned Writer"   } ,
+   {NOV, 16, "International Day for Tolerance"   } ,
+   {NOV, 17, "International Students Day"   } ,
+   {NOV, 20, "Universal Children's Day"   } ,
+   {NOV, 21, "World Television Day"   } ,
+   {NOV, 21, "World Fisheries Day"   } ,
+   {NOV, 21, "World Hello Day"   } ,
+   {NOV, 30, "Computer Security Day"   } ,
+   {DEC,  9, "International Anti-Corruption Day"   } ,
+   {DEC, 10, "Human Rights Day"   } ,
+   {DEC, 11, "International Mountain Day"   } ,
+   {DEC, 13, "International Day Day"   } ,
+   {DEC, 18, "International Migrants Day"   } ,
+
    /* cf. http://www.theonlinerecipebox.com/national-food-holidays/ */
 
    {JAN, 1, "Celebrate National Soup Month" } ,
@@ -863,7 +989,7 @@ static mday holiday[] = {
    {MAR, 30, "Celebrate National Peanut Month" } ,
    {MAR, 31, "National Tater Day" } ,
    {APR, 1, "National Sourdough Bread Day" } ,
-   {APR, 2, "National Peanut butter and Jelly Day" } ,
+   {APR, 2, "National Peanut Butter and Jelly Day" } ,
    {APR, 3, "National Chocolate Mousse Day" } ,
    {APR, 4, "National Cordon Bleu Day" } ,
    {APR, 5, "National Raisin and Spice Bar Day" } ,
@@ -878,7 +1004,7 @@ static mday holiday[] = {
    {APR, 14, "National Pecan Day" } ,
    {APR, 15, "National Glazed Spiral Ham Day" } ,
    {APR, 16, "National Eggs Benedict Day" } ,
-   {APR, 17, "National Cheese ball Day" } ,
+   {APR, 17, "National Cheese Ball Day" } ,
    {APR, 18, "Animal Crackers Birthday" } ,
    {APR, 19, "National Garlic Day" } ,
    {APR, 20, "National Pineapple Upside-down Cake Day" } ,
@@ -1222,6 +1348,16 @@ char * AFNI_get_date_trivia(void)
 
    /**** Special days not on fixed dates ****/
 
+   /* Day of month = 15 and month = 2 */
+
+   if( ntar < NTMAX && lt->tm_mday == 15 && lt->tm_mon+1 == FEB ){
+     static char aniv[64] ; int yy = lt->tm_year-95 ;
+     if( yy > 0 ){
+       sprintf(aniv,"Anniversary of AFNI's release: #%d",yy) ;
+       tar[ntar++] = aniv ;
+     }
+   }
+
    /* Day of month = 13 and weekday = Friday */
 
    if( ntar < NTMAX && lt->tm_mday == 13 && lt->tm_wday == 5 )
