@@ -17,6 +17,13 @@
 # define IS_GOOD_FLOAT(x) finite(x)
 #endif
 
+#undef  USE_SVDLIB
+#ifdef  USE_SVDLIB
+#include "svdlib.c"
+#endif
+
+void svd_double_ata( int m, int n, double *a, double *s, double *u, double *v ) ;
+
 /*---------------------------------------------------------------------------*/
 
 #undef  SQR
@@ -131,7 +138,6 @@ void symeig_3( double *a , double *e , int dovec )
 
    if( qq <= 0.0 ){       /*** This should never happen!!! ***/
      static int nerr=0 ;
-#pragma omp critical (STDERR)
      {
      if( ++nerr < 4 )
        fprintf(stderr,"** ERROR in symeig_3: discrim=%g numer=%g\n",qq,rr) ;
@@ -519,7 +525,6 @@ int first_principal_vectors( int n , int m , float *xx ,
 
    if( nvec > nsym ) nvec = nsym ;  /* can't compute more vectors than nsym! */
 
-#pragma omp critical (MALLOC)
    { asym  = (double *)malloc(sizeof(double)*nsym*nsym) ;  /* symmetric matrix */
      deval = (double *)malloc(sizeof(double)*nsym) ;       /* its eigenvalues */
    }
@@ -543,7 +548,6 @@ int first_principal_vectors( int n , int m , float *xx ,
    } else {                             /* more columns than rows:  */
                                         /* so [A] = [X][X]' = n x n */
      float *xt ; int m1=mm-1 ;
-#pragma omp critical (MALLOC)
      xt = (float *)malloc(sizeof(float)*nn*mm) ;
      for( jj=0 ; jj < mm ; jj++ ){      /* form [X]' into array xt */
        for( ii=0 ; ii < nn ; ii++ ) xt[jj+ii*mm] = xx[ii+jj*nn] ;
@@ -559,7 +563,6 @@ int first_principal_vectors( int n , int m , float *xx ,
        }
      }
 
-#pragma omp critical (MALLOC)
      free(xt) ;  /* don't need this no more */
    }
 
@@ -569,7 +572,6 @@ int first_principal_vectors( int n , int m , float *xx ,
    ii = symeig_irange( nsym, asym, deval, nsym-nvec, nsym-1, (uvec==NULL) ) ;
 
    if( ii != 0 ){
-#pragma omp critical (MALLOC)
      { free(deval) ; free(asym) ; }
      return -33333 ;  /* eigensolver failed!? */
    }
@@ -588,7 +590,6 @@ int first_principal_vectors( int n , int m , float *xx ,
    /** if no output vectors desired, we are done done done!!! **/
 
    if( uvec == NULL ){
-#pragma omp critical (MALLOC)
      { free(deval) ; free(asym) ; }
      return nvec ;
    }
@@ -635,7 +636,6 @@ int first_principal_vectors( int n , int m , float *xx ,
 
    /** free at last!!! **/
 
-#pragma omp critical (MALLOC)
    { free(deval) ; free(asym) ; }
    return nvec ;
 }
@@ -644,7 +644,7 @@ int first_principal_vectors( int n , int m , float *xx ,
 
 /*--------------------------------------------------------------------*/
 
-#define CHECK_SVD
+#define CHECK_SVD   /* unfortunately, this is necessary */
 
 #undef CHK
 #ifdef CHECK_SVD
@@ -721,13 +721,15 @@ void svd_double( int m, int n, double *a, double *s, double *u, double *v )
    (void) svd_( &mm , &nn , &lda , aa , ww ,
                 &matu , &ldu , uu , &matv , &ldv , vv , &ierr , rv1 ) ;
 
+   /** the following code is to check if the SVD worked **/
+
 #ifdef CHECK_SVD
    /** back-compute [A] from [U] diag[ww] [V]'
        and see if it is close to the input matrix;
        if not, compute the results in another function;
        this is needed because the svd() function compiles with
        rare computational errors on some compilers' optimizers **/
-   { register int i,j,k ; register doublereal aij ; double err=0.0,amag=1.e-11 ;
+   { register int i,j,k ; register doublereal aij ; double err=0.0,amag=1.e-12 ;
      for( j=0 ; j < n ; j++ ){
       for( i=0 ; i < m ; i++ ){
         aij = A(i,j) ; amag += fabs(aij) ;
@@ -737,16 +739,20 @@ void svd_double( int m, int n, double *a, double *s, double *u, double *v )
      amag /= (m*n) ; /* average absolute value of matrix elements */
      err  /= (m*n) ; /* average absolute error per matrix element */
      if( err >= 1.e-5*amag || !IS_GOOD_FLOAT(err) ){
-       fprintf(stderr,"SVD avg err=%g; recomputing ...",err) ;
+       fprintf(stderr,"\n **** SVD avg err=%g; recomputing ...",err) ;
 
-#if 1     /* mangle all zero columns */
-       { double arep=1.e-11*amag , *aj ;
+#if 1     /* perturb matrix slightly */
+       { double arep=1.e-13*amag , *aj ;
          for( j=0 ; j < nn ; j++ ){
            aj = aa + j*mm ;
+#if 1
+           for( i=0 ; i < mm ; i++ ) aj[i] += (drand48()-0.5)*arep ;
+#else
            for( i=0 ; i < mm ; i++ ) if( aj[i] != 0.0 ) break ;
            if( i == mm ){
              for( i=0 ; i < mm ; i++ ) aj[i] = (drand48()-0.5)*arep ;
            }
+#endif
          }
        }
 #endif
@@ -763,8 +769,34 @@ void svd_double( int m, int n, double *a, double *s, double *u, double *v )
           err += fabs(aij) ;
        }}
        err /= (m*n) ;
-       fprintf(stderr," new avg err=%g %s\n",
-               err , (err >= 1.e-5*amag || !IS_GOOD_FLOAT(err)) ? "**BAD**" : "**OK**" ) ;
+
+       /* not fixed YET?  try another algorithm (one that's usually slower) */
+
+       if( err >= 1.e-5*amag || !IS_GOOD_FLOAT(err) ){
+         fprintf(stderr," new avg err=%g; re-recomputing the hard way ...",err) ;
+#ifdef USE_SVDLIB
+         fprintf(stderr,"\n") ;
+         SVDVerbosity = 2 ;
+         AFNI_svdLAS2( mm , nn , aa , ww , uu , vv ) ;  /* svdlib.c */
+         SVDVerbosity = 0 ;
+#else
+         svd_double_ata( mm , nn , aa , ww , uu, vv ) ; /* below */
+#endif
+         err = 0.0 ;
+         for( j=0 ; j < n ; j++ ){
+          for( i=0 ; i < m ; i++ ){
+            aij = A(i,j) ;
+            for( k=0 ; k < n ; k++ ) aij -= U(i,k)*V(j,k)*ww[k] ;
+            err += fabs(aij) ;
+         }}
+         err /= (m*n) ;
+         fprintf(stderr," newer avg err=%g %s" ,
+                 err ,
+                 (err >= 1.e-5*amag || !IS_GOOD_FLOAT(err)) ? "**BAD** :-(" : "**OK** :-)" ) ;
+       } else {
+         fprintf(stderr," new avg error=%g **OK** :-)",err) ;
+       }
+       fprintf(stderr,"\n\n") ;
      }
    }
 #endif
@@ -788,22 +820,22 @@ void svd_double( int m, int n, double *a, double *s, double *u, double *v )
      qsort_doubleint( n , sv , iv ) ;
      if( u != NULL ){
        double *cc = (double *)calloc(sizeof(double),m*n) ;
-#pragma omp critical (MEMCPY)
+#pragma omp critical
        (void)memcpy( cc , u , sizeof(double)*m*n ) ;
        for( jj=0 ; jj < n ; jj++ ){
          kk = iv[jj] ;  /* where the new jj-th col came from */
-#pragma omp critical (MEMCPY)
+#pragma omp critical
          (void)memcpy( u+jj*m , cc+kk*m , sizeof(double)*m ) ;
        }
        free((void *)cc) ;
      }
      if( v != NULL ){
        double *cc = (double *)calloc(sizeof(double),n*n) ;
-#pragma omp critical (MEMCPY)
+#pragma omp critical
        (void)memcpy( cc , v , sizeof(double)*n*n ) ;
        for( jj=0 ; jj < n ; jj++ ){
          kk = iv[jj] ;
-#pragma omp critical (MEMCPY)
+#pragma omp critical
          (void)memcpy( v+jj*n , cc+kk*n , sizeof(double)*n ) ;
        }
        free((void *)cc) ;
@@ -816,6 +848,69 @@ void svd_double( int m, int n, double *a, double *s, double *u, double *v )
    return ;
 }
 
+/*----------------------------------------------------------------------------*/
+/* SVD via eigensolution of A'A (presumes n < m) */
+
+void svd_double_ata( int m, int n, double *a, double *s, double *u, double *v )
+{
+   double *ata , *ai,*aj , sum ;
+   int ii,jj,kk ;
+
+   if( a == NULL || s == NULL || m < 1 || n < 1 ) return ;
+
+   /* make A'A matrix */
+
+   ata = (double *)malloc(sizeof(double)*n*n) ;
+   for( ii=0 ; ii < n ; ii++ ){
+     ai = a + ii*m ;                   /* ii-th column of A */
+     for( jj=0 ; jj <= ii ; jj++ ){
+       aj = a + jj*m ;                 /* jj-th column of A */
+       for( sum=0.0,kk=0 ; kk < m ; kk++ ) sum += ai[kk]*aj[kk] ;
+       ata[ii+jj*n] = sum ; if( jj < ii ) ata[jj+ii*n] = sum ;
+     }
+   }
+
+   /* Since A = U S V', then A'A = V S^2 V', or A'A V = V S^2,
+      so the eigensolution of A'A gives S^2 as the eigenvalues,
+      and gives V as the eigenvectors (which here over-write ata). */
+
+   symeig_double( n , ata , s ) ;
+
+   /* sqrt the eigenvalues to get the singular values */
+
+   for( ii=0 ; ii < n ; ii++ )
+     s[ii] = (s[ii] <= 0.0) ? 0.0 : sqrt(s[ii]) ;
+
+   /* copy ata into the output place for V */
+
+   if( v != NULL ){
+#pragma omp critical
+     (void)memcpy( v , ata , sizeof(double)*n*n ) ;
+   }
+
+   /* manufacture U from A and V (which is in ata, recall):
+      AV = US, so if we form the matrix product AV, then
+      L2-normalizing each column will give us the matrix U. */
+
+   if( u != NULL ){
+     double *uj , *vj ;
+     for( jj=0 ; jj < n ; jj++ ){
+       uj = u   + jj*m ;              /* jj-th column of U */
+       vj = ata + jj*n ;              /* jj-th column of V */
+       for( ii=0 ; ii < m ; ii++ ){             /* A_{ii,kk} V_{kk,jj} */
+         for( sum=0.0,kk=0 ; kk < n ; kk++ ) sum += a[ii+kk*m]*vj[kk] ;
+         uj[ii] = sum ;
+       }
+       for( sum=0.0,ii=0 ; ii < m ; ii++ ) sum += uj[ii]*uj[ii] ;
+       if( sum > 0.0 ){
+         for( sum=1.0/sqrt(sum),ii=0 ; ii < m ; ii++ ) uj[ii] *= sum ;
+       }
+     }
+   }
+
+   free(ata) ; return ;
+}
+
 /*--------------------------------------------------------------------*/
 
 #define DBG_PC 0
