@@ -1244,6 +1244,7 @@ MCW_textwin * new_MCW_textwin( Widget wpar, char *msg, int type )
 static int bigtext = 0 ;
 void MCW_textwin_setbig( int b ){ bigtext = b ; }  /* 29 Apr 2009 */
 
+
 /*-----------------------------------------------------------------------
    Modified 10 Jul 2001 to include killing callback
 -------------------------------------------------------------------------*/
@@ -1739,7 +1740,8 @@ ENTRY("RWC_xineramize") ;
          if( xdef != NULL ){
             char *qdef = strdup(xdef) ;
             for( nn=0 ; qdef[nn] != '\0' ; nn++ )
-              if( qdef[nn] == '_' || qdef[nn] == ':' ) qdef[nn] = ' ' ;
+              if( qdef[nn] == '_' || qdef[nn] == ':' ||
+                  qdef[nn] == ';' || qdef[nn] == ','   ) qdef[nn] = ' ' ;
 
             nn = 0 ; sscanf(qdef,"%d%n",&nxsi,&nn) ;  /* number of sub-screens */
             if( nn <= 0 || nxsi <= 1 ){               /* ERROR */
